"""Clock Layer"""
from adafruit_datetime import date
from adafruit_display_text.label import Label
from adafruit_ntp import NTP
from displayio import Group
from os import getenv
from socketpool import SocketPool
from rtc import RTC
from terminalio import FONT

WHITE = 0xFFFFFF
MONTHS = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug",
          "Sep", "Oct", "Nov", "Dec")


class Clock:
    """Display and handle clock layer."""
    def __init__(self,
                 font: FONT,
                 group: Group,
                 pool: SocketPool,
                 display_width: int,
                 display_height: int):
        """Initialize.

        :param Font font: Clock font
        :param Group group: Display IO group to contain clock
        :param SocketPool pool: Socket pool for NTP
        :param int display_width: Width of OLED display
        :param int display_height: Height of OLED display
        """
        self._font = font
        self._group = group
        self._display_width = display_width
        self._display_height = display_height
        self._pool = pool
        self._visible = False
        self._offset = 0
        self._scale = 2  # 1=Small 2=Big
        # Initialize ESP32-S2 built-in real time clock
        self._rtc = RTC()
        self._last_time = ""
        self._ntp_up_to_date = False
        self.sync_time()  # Sync real time clock to NTP server
        # Using label instead of bitmap_label to reduce flicker on updates
        self._clock_label = Label(self._font,
                                  color=WHITE,
                                  x=0,
                                  y=0)
        self.parse_time()
        self._place_clock()

    @property
    def visible(self) -> bool:
        """Is the clock layer visible."""
        return self._visible

    @visible.setter
    def visible(self, show: bool) -> None:
        """Set clock layer visibility"""
        self._visible = show
        if show:
            if self._clock_label not in self._group:
                self._group.append(self._clock_label)
        else:
            if self._clock_label in self._group:
                self._group.remove(self._clock_label)

    @property
    def offset(self) -> int:
        """Horizontal offset."""
        return self._offset

    @offset.setter
    def offset(self, length: int) -> None:
        """Set horizontal offset"""
        self._offset = length
        self._place_clock()

    @property
    def scale(self) -> int:
        """Clock scale."""
        return self._scale

    @scale.setter
    def scale(self, level: int) -> None:
        """Set clock scale"""
        assert 1 <= level <= 2
        self._scale = level
        self._place_clock()

    def _place_clock(self) -> None:
        """Set clock placement on OLED"""
        if self._scale == 1:
            # Show small clock
            self._clock_label.scale = 1
            self._clock_label.x = (self._display_width // 2) + self._offset
            self._clock_label.y = 4
        elif self._scale == 2:
            # Show big clock
            self._clock_label.scale = 2
            self._clock_label.x = (self._display_width -
                                   (len(self._clock_label.text)
                                    * 12)) // 2 + self._offset
            self._clock_label.y = 30

    def is_dst(self, year: int, month: int, day: int) -> bool:
        """Check if in local daylight savings.
        :param int year: Current year
        :param int month: Current month
        :param int day: Current day
        NOTE:  The formula below is for most of North America.
               It would need to be modified for other areas.
        """
        # DST starts on 2nd Sunday of March (North America Only)
        dst_start = date(year, 3, 14 - date(year, 3, 1).weekday())
        # DST ends on 1st Sunday of November (North America Only)
        dst_end = date(year, 11, 7 - date(year, 11, 1).weekday())
        return dst_start <= date(year, month, day) <= dst_end

    def sync_time(self) -> None:
        """Synchronize real time clock with NTP server."""
        try:
            # Make sure you set timezone environment variable to your area
            ntp = NTP(self._pool, tz_offset=int(getenv("TIMEZONE")))

            # Adjust for daylight savings time
            # Comment out next 2 statements if you don't practice DST
            if self.is_dst(ntp.datetime.tm_year,
                           ntp.datetime.tm_mon,
                           ntp.datetime.tm_mday):
                # Assumes a 1 hour DST difference (modify as necessary)
                ntp = NTP(self._pool, tz_offset=int(getenv("TIMEZONE")) + 1)

            self._rtc.datetime = ntp.datetime
            self._ntp_up_to_date = True
        except Exception as err:
            # Need error check because occasional bug in NTP code
            print(f"Sync time error: {err}")
            self._ntp_up_to_date = False

    def parse_time(self) -> int:
        """Parse formatted date/time data."""
        if not self._ntp_up_to_date:  # Sync with NTP if not up-to-date
            self.sync_time()

        t = self._rtc.datetime
        h = t.tm_hour

        if h >= 12:
            meridiem = "PM"
        else:
            meridiem = "AM"

        if h > 12:
            h -= 12  # Convert military to standard time

        # Use lowercase to designate RTC has not been synchronized
        if not self._ntp_up_to_date:
            meridiem = meridiem.lower()

        new_time = (f'{MONTHS[t.tm_mon - 1]} {t.tm_mday:02d}  ' +
                    f'{h}:{t.tm_min:02d} {meridiem}')

        # Only update display when time changes to avoid flicker & save memory
        if self._last_time != new_time:
            self._clock_label.text = new_time
            self._last_time = new_time
        return t.tm_hour  # Return hour to track time (military time)