from adafruit_bitmap_font import bitmap_font
import adafruit_minimqtt.adafruit_minimqtt as mqtt
import adafruit_ssd1322
from board import D6, D7, D9, D17, D18, RX, TX, SPI
import displayio
import gc
import microcontroller
from os import getenv
from socketpool import SocketPool  # For MQTT & NTP
import ssl
import supervisor
import terminalio
from time import sleep
import wifi
from clock_layer import Clock
from disconnection_layer import Disconnection
from message_layer import Message
from password_layer import Password
from mqtt_manager import MQTTManager

# Generate a unique 19-bit WiFi client hostname using CPU UID
client_id = 'QTPY' + str(int.from_bytes(microcontroller.cpu.uid, 'little')
                         >> 29)
if wifi.radio.hostname != client_id:
    wifi.radio.hostname = client_id
print(f'WiFi Hostname: {wifi.radio.hostname}')

# Ensure that Display IO releases any existing displays
displayio.release_displays()

# Initialize SSD1322 OLED display
spi = SPI()  # SPI pins labeled SCK, MOSI (MISO not used)
dc = D9  # D9 labeled A2
cs = D18  # D18 labeled A0
reset = D17  # D17 labeled A1
display_bus = displayio.FourWire(spi, command=dc, chip_select=cs,
                                 reset=reset, baudrate=5000000)
display = adafruit_ssd1322.SSD1322(display_bus, width=256, height=64,
                                   colstart=112)

# Initialize group to hold all display graphics
group = displayio.Group()

small_font = terminalio.FONT
large_font = bitmap_font.load_font("/profont22.bdf")


def connect_to_wifi():
    """Connect to Wi-Fi."""
    print(f'Connecting to {getenv("CIRCUITPY_WIFI_SSID")}...')
    while not wifi.radio.ipv4_address:
        try:
            wifi.radio.connect(getenv('CIRCUITPY_WIFI_SSID'),
                               getenv('CIRCUITPY_WIFI_PASSWORD'))
        except (ConnectionError) as con_err:
            print("Connection Error:", con_err)
            print("Retrying in 10 seconds")
        sleep(10)
    print(f'Connected to {getenv("CIRCUITPY_WIFI_SSID")}')


if not wifi.radio.ipv4_address:
    connect_to_wifi()

# Initialize socket pool
pool = SocketPool(wifi.radio)

# Initialize clock
clock_layer = Clock(font=small_font,
                    group=group,
                    pool=pool,
                    display_width=display.width,
                    display_height=display.height)

# Initialize disconnection layer
disconnection_layer = Disconnection(group=group)

# Initialize message layer
message_layer = Message(bit_clock=TX,
                        word_select=RX,
                        data=D6,
                        amp_enable=D7,
                        font=large_font,
                        group=group)

# Initialize Wi-Fi password layer
password_layer = Password(small_font=small_font,
                          large_font=large_font,
                          group=group,
                          display_width=display.width,
                          display_height=display.height)

def message(message):
    """Callback fired when topic has new message."""
    clean_text = "".join(c.upper() for c in message if c.isalpha())
    if clean_text == "WIFI":
        # Adjust clock scale so it doesn't block Wi-Fi password
        clock_layer.scale = 1
        # Display Wi-Fi QR code received
        password_layer.visible = True  # Show Wi-Fi password text & QR code
    elif clean_text == "OFF":
        # Clear display
        password_layer.visible = False
        clock_layer.scale = 2
    elif clean_text == "RESET":
        # Reboot the microcontroller
        microcontroller.reset()  # Same as pressing reset button
    else:
        # Display SMS text message
        message_layer.show_message(message=message, delay=10)

# Set up a MiniMQTT Client
mqtt_client = mqtt.MQTT(
    client_id=client_id,
    broker=getenv("BROKER"),
    port=int(getenv("PORT")),
    username=getenv("AIO_USERNAME"),
    password=getenv("AIO_KEY"),
    socket_pool=pool,
    ssl_context=ssl.create_default_context(),
)

# Initialize MQTT manager and connect
mqtt_manager = MQTTManager(mqtt_client, message)
mqtt_manager.connect()

last_hour = -1
clock_layer.visible = True  # Show clock
display.show(group)  # Show DisplayIO group on OLED display

# Main program loop
try:
    while True:
        # Update clock
        hour = clock_layer.parse_time()

        # Hourly checks
        if hour != last_hour:
            # Soft reload daily to address CircuitPython bugs
            if hour == 0 and last_hour == 23:
                supervisor.reload()
            last_hour = hour
            print(f'Free memory: {gc.mem_free()}, hour: {hour}')

            # Adjust Wi-Fi QR / password X position to avoid burn-in
            password_layer.offset = hour - (4 * (hour // 4))

            # Adjust clock position to avoid burn-in
            clock_layer.offset = hour - 12

            # Hide displayed Wi-Fi password if outside business hours
            if hour < 5 or hour > 17:
                password_layer.visible = False
                clock_layer.scale = 2

            # Show or hide clock depending on extended business hours
            if 5 <= hour <= 21:
                clock_layer.visible = True
            else:
                clock_layer.visible = False

        # Check Wi-Fi is connected
        if wifi.radio.ipv4_address is None:
            mqtt_manager.status = "DISCONNECTED"
            disconnection_layer.visible = True
            connect_to_wifi()
            continue

        if mqtt_manager.loop():
            disconnection_layer.visible = False

        gc.collect()
        sleep(1)
except KeyboardInterrupt:
    print("\nCtrl-C pressed.  Cleaning up...")
finally:
    display.sleep()
    displayio.release_displays()
    mqtt_client.deinit()
    message_layer.deinit()
    spi.deinit()