"""Message Layer"""
from adafruit_bitmap_font.bdf import BDF
from adafruit_display_text import bitmap_label, wrap_text_to_lines
import audiobusio
import audiocore
from digitalio import DigitalInOut, Direction
from displayio import Group
from microcontroller import Pin
from time import sleep


class Message:
    """Display and handle SMS messages."""
    def __init__(self,
                 bit_clock: Pin,
                 word_select: Pin,
                 data: Pin,
                 amp_enable: Pin,
                 font: BDF,
                 group: Group):
        """Initialize.

        :param Pin bit_clock: Board GPIO pin for I2S bit clock
        :param Pin word_select: Board GPIO pin for I2S word select
        :param Pin data: Board GPIO pin for I2S data
        :param Pin amp_enable: Board GPIO pin for I2S amp enable
        :param BDF font: Message font
        :param Group group: Display IO group to contain message
        """
        # Initialize I2S Amplifier
        self._audio = audiobusio.I2SOut(bit_clock=bit_clock,
                                        word_select=word_select,
                                        data=data)

        # Initialize notification bell sound file
        self._bell = audiocore.WaveFile("bell2x.wav")

        # Initialize amplifier enable/disable pin
        self._amp_enable = DigitalInOut(amp_enable)
        self._amp_enable.direction = Direction.OUTPUT
        self._amp_enable.value = False

        self._group = group

        # Initialize message label
        self._message_label = bitmap_label.Label(font,
                                                 anchor_point=(0, 0),
                                                 anchored_position=(0, 0),
                                                 line_spacing=1.0)

    def show_message(self, message: str, delay: int) -> None:
        """Show the message for specified delay in seconds.

        :param str message: The message to display
        :param int delay: The delay in seconds to show message
        """
        # Confirm message not already displayed
        if self._message_label in self._group:
            return

        # Message formatting
        message = message.replace(" and ", " & ")  # Abbreviate and
        self._message_label.text = "\n".join(wrap_text_to_lines(message, 21))

        # Store Display IO group state and clear
        prior_state = []
        for i in range(len(self._group)):
            prior_state.append(self._group.pop())

        # Display message
        self._group.append(self._message_label)

        # Play notification bell sound
        self._amp_enable.value = True
        self._audio.play(self._bell)
        while self._audio.playing:
            pass
        self._amp_enable.value = False

        sleep(delay)

        # Clear message
        self._group.remove(self._message_label)

        # Restore prior Display IO group state
        for i in range(len(prior_state)):
            self._group.append(prior_state[i])

    def deinit(self) -> None:
        """Deinitialize class."""
        self._bell.deinit()
        self._audio.deinit()
