import adafruit_minimqtt.adafruit_minimqtt as mqtt
from os import getenv

class MQTTManager:
    """Handle MQTT communications."""
    def __init__(self,
                 mqtt_client: mqtt.MQTT,
                 onmessage: callable):
        """Initialize.

        :param MQTT mqtt_client: MQTT Client
        :param callable onmessage: Callback for subscribed messages
        """
        self.mqtt_client = mqtt_client
        self.aio_feed = getenv("AIO_FEED")
        self.onmessage = onmessage
        self.status = "DISCONNECTED"  # Used to track MQTT connection status
        # Wire the callback methods
        self.mqtt_client.on_connect = self.connected
        self.mqtt_client.on_disconnect = self.disconnected
        self.mqtt_client.on_message = self.message
        self.mqtt_client.on_subscribe = self.subscribed

    # Define callback methods for Adafruit IO MQTT feed
    def connected(self, client, userdata, flags, rc):
        """Fired when client successfully connected to broker."""
        self.status = "CONNECTED"
        print(f"Adafruit IO connected.  Client ID: {client.client_id}")
        # Subscribe to all changes on the feed.
        client.subscribe(self.aio_feed, qos=1)

    def disconnected(self, client, userdata, rc):
        """Fired when client is disconnected from broker."""
        self.status = "DISCONNECTED"
        print("Disconnected from Adafruit IO.")

    def message(self, client, topic, message):
        """Fired when topic has new message."""
        print(f'Topic: {topic}, message: {message}')
        self.onmessage(message)

    def subscribed(self, mqtt_client, userdata, topic, granted_qos):
        """Fired when the mqtt_client subscribes to a new feed."""
        print("Subscribed to {0} at QOS level {1}".format(topic, granted_qos))

    # Connect the client to the MQTT broker.
    def connect(self) -> None:
        # Only attempt connection if disconnected
        if self.status == "DISCONNECTED":
            self.status = "CONNECTING"
            print("Connecting to Adafruit IO...")
            try:
                self.mqtt_client.connect()
            except Exception as mqtt_connect_err:
                self.status = "DISCONNECTED"
                print(f"MQTT connection error: {mqtt_connect_err}")

    def loop(self) -> bool:
        try:
            if self.status == "CONNECTING":
                # Exit because still connecting
                print('Connecting')
                return False

            if self.status == "DISCONNECTED":
                # Reconnect to MQTT broker if disconnected
                self.status = "CONNECTING"
                try:
                    print("Attempting to reconnect to MQTT broker")
                    self.mqtt_client.reconnect(resub_topics=False)
                except Exception as reconnect_err:
                    print(f"MQTT broker reconnect failure: {reconnect_err}")
                    self.status = "DISCONNECTED"
                return False

            # Check for MQTT client connection disruptions
            if self.mqtt_client.is_connected():  # Throws error on false
                self.mqtt_client.loop()  # Check incoming subscription messages
                return True
        except mqtt.MMQTTException as mqtt_err:
            # MQTT Error
            print(f"MMQTTException: {mqtt_err}")
            return False
        except OSError as os_err:
            # OS Error (probably WLAN or TLS)
            print(f"OSError: {os_err}")
            # Disconnect MQTT to force Reconnect
            self.status = "DISCONNECTED"
            return False