"""Password Layer"""
from adafruit_bitmap_font.bdf import BDF
from adafruit_display_text.bitmap_label import Label
from adafruit_miniqr import L, QRCode
from displayio import Bitmap, Group, Palette, TileGrid
from os import getenv
from terminalio import FONT

WHITE = 0xFFFFFF
BLACK = 0x000000
BORDER_PIXELS = 2


class Password:
    """Display and handle Wi-Fi password layer."""
    def __init__(self,
                 small_font: FONT,
                 large_font: BDF,
                 group: Group,
                 display_width: int,
                 display_height: int):
        """Initialize.

        :param Font small_font: Wi-Fi password label font
        :param BDF large_font: Password font
        :param Group group: Display IO group to contain password
        :param int display_width: Width of OLED display
        :param int display_height: Height of OLED display
        """
        self._small_font = small_font
        self._large_font = large_font
        self._group = group
        self._display_width = display_width
        self._display_height = display_height
        self._visible = False
        self._offset = 0
        # Generate QR code for Wi-Fi
        qr = QRCode(qr_type=3, error_correct=L)
        # Format data for Wi-Fi connect
        qr.add_data(f"WIFI:S:{getenv('CIRCUITPY_WIFI_SSID')};T:WPA;P:" +
                    f"{getenv('CIRCUITPY_WIFI_PASSWORD')};;")
        qr.make()

        # Generate the 1-pixel-per-bit bitmap
        qr_bitmap = self._bitmap_qr(qr.matrix)
        self._bitmap_width = qr_bitmap.width

        # Scale the QR code as big as the display can handle
        scale = min(self._display_width // qr_bitmap.width,
                    self._display_height // qr_bitmap.height)
        pos_y = int(((self._display_height / scale) - qr_bitmap.height) / 2)

        # Define palette for QR codes
        qr_palette = Palette(2)
        qr_palette[0] = WHITE
        qr_palette[1] = BLACK

        # Initialize tile grid to contain QR code
        self._qr_code = TileGrid(qr_bitmap,
                                 pixel_shader=qr_palette,
                                 x=0,
                                 y=pos_y)

        # Initialize Wi-Fi label
        self._wifi_label = Label(self._small_font,
                                 text=(f"{getenv('CIRCUITPY_WIFI_SSID')}" +
                                       " Wi-Fi Password:"),
                                 color=WHITE,
                                 anchor_point=(0, 0),
                                 anchored_position=(qr_bitmap.width + 8, 16))
        # Initialize password label
        self._password_label = Label(self._large_font,
                                     text=getenv('CIRCUITPY_WIFI_PASSWORD'),
                                     color=WHITE,
                                     anchor_point=(0, 0),
                                     anchored_position=(qr_bitmap.width
                                                        + 8, 29))

    @staticmethod
    def _bitmap_qr(matrix):
        """ Generate Display IO bitmap from QR matrix"""
        # Define bitmap to hold QR code
        bitmap = Bitmap(matrix.width + 2 * BORDER_PIXELS,
                        matrix.height + 2 * BORDER_PIXELS,
                        2)
        # Raster the QR code
        for y in range(matrix.height):  # each scanline in the height
            for x in range(matrix.width):
                if matrix[x, y]:
                    bitmap[x + BORDER_PIXELS, y + BORDER_PIXELS] = 1
                else:
                    bitmap[x + BORDER_PIXELS, y + BORDER_PIXELS] = 0
        return bitmap

    @property
    def visible(self) -> bool:
        """Is the password layer visible."""
        return self._visible

    @visible.setter
    def visible(self, show: bool) -> None:
        """Set password layer visibility"""
        self._visible = show
        if show:
            if self._qr_code not in self._group:
                self._group.append(self._qr_code)
            if self._wifi_label not in self._group:
                self._group.append(self._wifi_label)
            if self._password_label not in self._group:
                self._group.append(self._password_label)
        else:
            if self._qr_code in self._group:
                self._group.remove(self._qr_code)
            if self._wifi_label in self._group:
                self._group.remove(self._wifi_label)
            if self._password_label in self._group:
                self._group.remove(self._password_label)

    @property
    def offset(self) -> int:
        """Horizontal offset."""
        return self._offset

    @offset.setter
    def offset(self, length: int) -> None:
        """Set horizontal offset"""
        self._offset = length
        self._qr_code.x = self._offset
        self._wifi_label.anchored_position = (self._bitmap_width
                                              + 8 + self._offset, 16)
        self._password_label.anchored_position = (self._bitmap_width
                                                  + 8 + self._offset, 29)